<?php

App::uses('AppBehavior', 'Model/Behavior');

class TemplateBehavior extends AppBehavior {

	/**
	 * Setup this behavior with the specified configuration settings.
	 *
	 * @param Model $model Model using this behavior
	 * @param array $config Configuration settings for $model
	 * @return void
	 */
	public function setup(Model $Model, $config = array()) {
		$Model->addBehaviour('EvCustomFields.CustomFieldsData', array('formInject' => true));
	}

	/**
	 * get the image / document blocks for the selected template
	 * then associate the model wityh the image / docs so we can save
	 *
	 * @param Model $model Model using this behavior
	 * @param array $options Options passed from Model::save().
	 * @return mixed False if the operation should abort. Any other result will continue.
	 * @see Model::save()
	 */
	public function afterValidate(Model $Model) {
		$return = parent::afterValidate($Model);

		// when calling from within behavior we need to manually add the Model var in.
		if (isset($Model->data[$Model->alias]['template_id']) && ! empty($Model->data[$Model->alias]['template_id'])) {
			$this->setupAttachmentRelationships($Model, $Model->data[$Model->alias]['template_id']);
		}

		return $return;
	}

	/**
	 * given the template id setup the image / docs relationships
	 *
	 * @param   Model $model Model using this behavior
	 * @param   int     templateId
	 * @return  void
	 */
	public function setupAttachmentRelationships($Model, $templateId) {
		$imgBlocks = $this->getAttachmentBlocks('EvTemplates.ImageBlock', $templateId);
		$docBlocks = $this->getAttachmentBlocks('EvTemplates.DocumentBlock', $templateId);

		$this->setupBlock($Model, 'imageSlots', 'ImageBlock', $imgBlocks);
		$this->setupBlock($Model, 'documentSlots', 'DocumentBlock', $docBlocks);

		$Model->initialiseAssociateModels();
	}

	/**
	 * setup a single attachment block
	 *
	 * @param   Model $model Model using this behavior
	 * @param   string  Type of block imageSlots|documentSlots
	 * @param   string  Model Alias ImageBlock|DocumentBlock
	 * @param   array   Array of blocks
	 * @return  void
	 */
	public function setupBlock($Model, $type, $modelAlias, $blocks) {
		if (! empty($blocks)) {
			$slots = array();

			foreach ($blocks as $block) {
				$slots[$block[$modelAlias]['name']] = array(
					'slots' => $block[$modelAlias]['slots'],
					'fields' => $block[$modelAlias]['blockFields']
				);
			}
			$Model->$type = $slots;
		}
	}

	/**
	 * get image and document blocks
	 *
	 * @param   string  Attachment type
	 * @param   int     templateId
	 * @return  array|   array of image blocks / document blocks
	 */
	public function getAttachmentBlocks($attachmentType, $templateId) {
		$attachmentModel = EvClassRegistry::init($attachmentType);
		$blocks = $attachmentModel->find(
			'all',
			array(
				'conditions' => array(
					$attachmentModel->alias . '.is_active' => 1,
					$attachmentModel->alias . '.template_id' => $templateId
				)
			)
		);

		if (! empty($blocks)) {
			return $blocks;
		} else {
			return false;
		}
	}

	/**
	 * get images for an item retrieved not via readForEdits
	 *
	 * @param   object          Model Object - Cake handles this
	 * @param   array           array of item data
	 * @param   string|array    image block(s) to retrieve
	 * @param   string          model alias
	 * @return  array
	 */
	public function getImages($Model, $data, $blocks, $modelAlias = null) {
		if (empty($modelAlias)) {
			$modelAlias = $Model->alias;
		}

		if (! isset($data[$modelAlias]['id']) || empty($data[$modelAlias]['id'])) {
			return array();
		}

		if (! is_array($blocks)) {
			$blocks = array($blocks);
		}

		$Image = EvClassRegistry::init('EvCore.Image');

		$images = $Image->find(
			'all',
			array(
				'conditions' => array(
					'Image.model' => $modelAlias,
					'Image.model_id' => $data[$modelAlias]['id'],
					'Image.attachment_type' => $blocks
				),
				'order' => 'Image.sequence ASC'
			)
		);

		if (empty($images)) {
			return array();
		}

		return Hash::combine($images, '{n}.Image.id', '{n}.Image', '{n}.Image.attachment_type');
	}

	/**
	 * get documents for an item retrieved not via readForEdits
	 *
	 * @param   object          Model Object - Cake handles this
	 * @param   array           array of item data
	 * @param   string|array    document block(s) to retrieve
	 * @param   string          model alias
	 * @return  array
	 */
	public function getDocuments($Model, $data, $blocks, $modelAlias = null) {
		if (empty($modelAlias)) {
			$modelAlias = $Model->alias;
		}

		if (! isset($data[$modelAlias]['id']) || empty($data[$modelAlias]['id'])) {
			return array();
		}

		if (! is_array($blocks)) {
			$blocks = array($blocks);
		}

		$Document = EvClassRegistry::init('EvCore.Document');

		$documents = $Document->find(
			'all',
			array(
				'conditions' => array(
					'Document.model' => $modelAlias,
					'Document.model_id' => $data[$modelAlias]['id'],
					'Document.attachment_type' => $blocks
				),
				'order' => 'Document.sequence ASC'
			)
		);

		if (empty($documents)) {
			return array();
		}

		return Hash::combine($documents, '{n}.Document.id', '{n}.Document', '{n}.Document.attachment_type');
	}

	/**
	 * beforeBeforeSave callback,
	 * run before saveAll is called, allowing you to modify the whole data array
	 * before save processes
	 *
	 * @param 	object 		Model - populated by Cakes ORM
	 * @param 	array 		Array to save
	 * @param 	array 		Options array
	 * @return 	array|bool 	Bool false to stop save, return modified data array to proceed
	 */
	public function beforeBeforeSave($Model, $data, $options = array()) {
		if (empty($data[$Model->alias]['template_id'])) {
			return $data;
		}

		foreach ($data as $key => $item) {
			if (! is_array($item)) {
				continue;
			}

			$attachmentType = Hash::extract($item, '{n}.attachment_type');
			if (empty($attachmentType)) {
				continue;
			}

			$data[$key] = Hash::insert($item, '{n}.template_id', $data[$Model->alias]['template_id']);
		}

		return $data;
	}
}
